@@ -24,6 +24,8 @@ CLOSURES_FILE = BASE_DIR + 'Closures.txt'
 CONFIG_FILE = 'Config.txt'
 
 MAP_IMAGE = os.path.dirname(argv[0]) + "/tfl_map.gif"
+MAP_SCALE = 2
+MAP_BUFFER = 10
 
 CLOSURES_TIMEOUT = 300
 
@@ -403,20 +405,20 @@ def parse(plan, indent=""):
 			return plural
 		else:
 			return singular
-	tmp = ""
+	tmp = []
 	if plan != None:
 		if len(plan[0]) > 1:
 			for i in range(len(plan[0])):
 				if i == 0:
-					tmp = "%s(%d) Take %s line (%s) from %s (%s) for %d stop%s.\n" % (indent, (i + 1), plan[1][0], plan[2][i], Station[STATION_NAME][plan[0][0]], zones(plan[0][0]), plan[3][0], suffix(plan[3][0]))
+					tmp.append("%s(%d) Take %s line (%s) from %s (%s) for %d stop%s." % (indent, (i + 1), plan[1][0], plan[2][i], Station[STATION_NAME][plan[0][0]], zones(plan[0][0]), plan[3][0], suffix(plan[3][0])))
 				elif i == len(plan[0]) - 1:
-					tmp += "%s(%d) Arrive at %s (%s) after %d minute%s." % (indent, (i + 1), Station[STATION_NAME][plan[0][i]], zones(plan[0][i]), plan[4], suffix(plan[4]))
+					tmp.append("%s(%d) Arrive at %s (%s) after %d minute%s." % (indent, (i + 1), Station[STATION_NAME][plan[0][i]], zones(plan[0][i]), plan[4], suffix(plan[4])))
 				else:
-					tmp += "%s(%d) Switch to %s line (%s) at %s and continue for %d stop%s.\n" % (indent, (i + 1), plan[1][i], plan[2][i], Station[STATION_NAME][plan[0][i]], plan[3][i], suffix(plan[3][i]))
+					tmp.append("%s(%d) Switch to %s line (%s) at %s and continue for %d stop%s." % (indent, (i + 1), plan[1][i], plan[2][i], Station[STATION_NAME][plan[0][i]], plan[3][i], suffix(plan[3][i])))
 		else:
-			tmp += "%sNo route possible" % indent
+			tmp.append("%sNo route possible" % indent)
 	else:
-		tmp += "%sNo route found" % indent
+		tmp.append("%sNo route found" % indent)
 	return tmp
 
 
@@ -449,17 +451,44 @@ def main():
 		pannable_area = hildon.PannableArea()
 		pannable_area.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
 		image = gtk.Image()
-		image.set_from_pixbuf(mapbuf)
 		image_event = gtk.EventBox()
-		image_event.add(image)
+		image_event.add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.KEY_PRESS_MASK)
 		if route.plan != None:
-			for stop in route.plan[0]:
-				#Station[STATION_COORDS][stop][0], Station[STATION_COORDS][stop][1], Station[STATION_COORDS][stop][2], Station[STATION_COORDS][stop][3]
-				pass
+			pixbuf = mapbuf
+			pixmap,mask = pixbuf.render_pixmap_and_mask()
+			cm = pixmap.get_colormap()
+			red = cm.alloc_color('red')
+			black = cm.alloc_color('black')
+			white = cm.alloc_color('white')
+			gc_red = pixmap.new_gc(foreground=red,line_width=5)
+			gc_black = pixmap.new_gc(foreground=black,line_width=5)
+			gc_white = pixmap.new_gc(foreground=white,line_width=9)
+			for i in range(len(route.plan[0])):
+				stop = route.plan[0][i]
+				if len(Station[STATION_COORDS][stop]) == 4:
+					w = Station[STATION_COORDS][stop][2] - Station[STATION_COORDS][stop][0]
+					h = Station[STATION_COORDS][stop][3] - Station[STATION_COORDS][stop][1]
+					x, y, dx, dy = int(Station[STATION_COORDS][stop][0]*MAP_SCALE - MAP_BUFFER), int(Station[STATION_COORDS][stop][1]*MAP_SCALE - MAP_BUFFER), int(w*MAP_SCALE + 2*MAP_BUFFER), int(h*MAP_SCALE + 2*MAP_BUFFER)
+					if i == 0:
+						map_x = int(x + dx/2)
+						map_y = int(y + dy/2)
+					if (i == 0) or (i == len(route.plan[0]) - 1):
+						gc_ = gc_red
+					else:
+						gc_ = gc_black
+					for gc in [gc_white, gc_]:
+						pixmap.draw_rectangle(gc, False, x, y, dx, dy)
+			image.set_from_pixmap(pixmap,mask)
+		else:
+			image.set_from_pixbuf(mapbuf)
 		if not do_route:
-			image_event.connect("button_press_event", mapclick, pannable_area)
+			image_event.connect("button_press_event", lambda widget,event,data: mapclick(widget,event,data,title,route), pannable_area)
 		else:
 			image_event.connect("button_press_event", lambda widget,event,data: maproute(widget,event,data,title,route), pannable_area)
+		#image_event.set_flags(gtk.CAN_FOCUS)
+		#image_event.grab_focus()
+		#image_event.connect("key_press_event", mapkey, pannable_area)
+		image_event.add(image)
 		image_event.oldtime = 0
 		image_event.container = win
 		win.start = ""
@@ -472,51 +501,74 @@ def main():
 			hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Choose starting point (double-click)...")
 
 	def maproute(widget, event, data, title, route):
+		event.x = event.x / MAP_SCALE
+		event.y = event.y / MAP_SCALE
 		if (event.time - widget.oldtime) < 250:
 			if (widget.container.start == "") or (widget.container.end == ""):
 				minimum = ["", INFINITY]
 				for station in Station[STATION_COORDS]:
 					if len(Station[STATION_COORDS][station]) == 4:
-						if (event.x >= Station[STATION_COORDS][station][0]) and (event.x <= Station[STATION_COORDS][station][2]) and (event.y >= Station[STATION_COORDS][station][1]) and (event.y <= Station[STATION_COORDS][station][3]):
-							minimum = [station, 0]
-						if minimum[1] > 0:
+						if (event.x >= Station[STATION_COORDS][station][0] - MAP_BUFFER) and (event.x <= Station[STATION_COORDS][station][2] + MAP_BUFFER) and (event.y >= Station[STATION_COORDS][station][1] - MAP_BUFFER) and (event.y <= Station[STATION_COORDS][station][3] + MAP_BUFFER):
 							dist = ((Station[STATION_COORDS][station][0] + Station[STATION_COORDS][station][2])/2. - event.x)**2 + ((Station[STATION_COORDS][station][1] + Station[STATION_COORDS][station][3])/2. - event.y)**2
 							if dist < minimum[1]:
 								minimum = [station, dist]
-				if widget.container.start == "":
-					widget.container.start = minimum[0]
-					hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s. Choose destination (double-click)..." % Station[STATION_NAME][minimum[0]])
-				else:
-					widget.container.end = minimum[0]
-					hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s." % Station[STATION_NAME][minimum[0]])
-					prompt = hildon.hildon_note_new_confirmation(widget.container, "Do you want to route from %s to %s?" % (Station[STATION_NAME][widget.container.start], Station[STATION_NAME][widget.container.end]))
-					response = gtk.Dialog.run(prompt)
-					prompt.destroy()
-					if response == gtk.RESPONSE_OK:
-						title.station_from = widget.container.start
-						title.station_to = widget.container.end
-						plan = guide(title.station_from, title.station_to)
-						plan[0] = plan[0].replace('&', '&amp;')
-						title.set_markup("<big>%s to %s</big>" % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)]))
-						route.get_buffer().set_text(plan[0])
-						route.plan = plan[1]
-						hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
-						widget.container.destroy()
-					elif response == gtk.RESPONSE_CANCEL:
-						widget.container.destroy()
+				if minimum[1] < INFINITY:
+					if widget.container.start == "":
+						widget.container.start = minimum[0]
+						hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s. Choose destination (double-click)..." % Station[STATION_NAME][minimum[0]])
 					else:
-						widget.container.start = ""
-						widget.container.end = ""
+					 	widget.container.end = minimum[0]
+						hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s." % Station[STATION_NAME][minimum[0]])
+						dialog = gtk.Dialog("Tubes", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
+						dialog.add_button("Yes", gtk.RESPONSE_OK)
+						dialog.add_button("No", gtk.RESPONSE_NO)
+						dialog.add_button("Cancel", -1)
+						label = gtk.Label("Do you want to route from %s to %s?" % (Station[STATION_NAME][widget.container.start], Station[STATION_NAME][widget.container.end]))
+						label.set_line_wrap(True)
+						dialog.vbox.pack_start(label, True, True, 0)
+						dialog.show_all()
+						response = dialog.run()
+						dialog.destroy()
+						if response == gtk.RESPONSE_OK:
+							title.station_from = widget.container.start
+							title.station_to = widget.container.end
+							plan = guide(title.station_from, title.station_to)
+							route.plan = plan[1]
+							route.guide = plan[0]
+							plan[0] = '\n'.join(plan[0])
+							s = "<big>%s to %s</big>" % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)])
+							title.set_markup(s.replace('&', '&amp;'))
+							route.get_buffer().set_text(plan[0])
+							hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
+							widget.container.destroy()
+						elif response == -1:
+							widget.container.destroy()
+						else:
+							widget.container.start = ""
+							widget.container.end = ""
 		widget.oldtime = event.time
 
-	def mapclick(widget, event, data):
+	def mapclick(widget, event, data, title, route):
 		if (event.time - widget.oldtime) < 250:
 			widget.container.destroy()
+		else:
+			if route.plan != None:
+				if len(route.plan[0]) > 1:
+					for i in range(len(route.plan[0]) - 1):
+						stop = route.plan[0][i]
+						if len(Station[STATION_COORDS][stop]) == 4:
+							w = Station[STATION_COORDS][stop][2] - Station[STATION_COORDS][stop][0]
+							h = Station[STATION_COORDS][stop][3] - Station[STATION_COORDS][stop][1]
+							x, y, dx, dy = int(Station[STATION_COORDS][stop][0]*MAP_SCALE - MAP_BUFFER), int(Station[STATION_COORDS][stop][1]*MAP_SCALE - MAP_BUFFER), int(w*MAP_SCALE + 2*MAP_BUFFER), int(h*MAP_SCALE + 2*MAP_BUFFER)
+							if (event.x >= x) and (event.x <= x + dx) and (event.y >= y) and (event.y <= y + dy):
+								stop = route.plan[0][i+1]
+								hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', route.guide[i])
+								data.scroll_to(int((Station[STATION_COORDS][stop][0] + Station[STATION_COORDS][stop][2])/2*MAP_SCALE), int((Station[STATION_COORDS][stop][1] + Station[STATION_COORDS][stop][3])/2*MAP_SCALE))
 		widget.oldtime = event.time
 
 	def showdialog(title, route):
 		hildon.hildon_gtk_window_set_progress_indicator(win, 1)
-		dialog = gtk.Dialog("Tube route", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
+		dialog = gtk.Dialog("Tubes", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
 		dialog.add_button("Get route", gtk.RESPONSE_OK)
 		dialog.add_button("Cancel", -1)
 		from_hbox = gtk.HBox()
@@ -546,41 +598,43 @@ def main():
 		title.station_to = to_entry.get_text().strip()
 		dialog.destroy()
 		if (response == gtk.RESPONSE_OK) and (title.station_from != "") and (title.station_to != ""):
-			title.set_markup("<big>%s to %s</big>" % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)]))
+			title.set_markup("<big>%s to %s</big>".replace('&', '&amp;') % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)]))
 			plan = guide(title.station_from, title.station_to)
-			plan[0] = plan[0].replace('&', '&amp;')
 			route.plan = plan[1]
+			route.guide = plan[0]
+			plan[0] = '\n'.join(plan[0])
 			route.get_buffer().set_text(plan[0])
 			hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
 		hildon.hildon_gtk_window_set_progress_indicator(win, 0)
 
 	mapbuf = gtk.gdk.pixbuf_new_from_file(MAP_IMAGE)
+	mapbuf = mapbuf.scale_simple(MAP_SCALE*mapbuf.get_width(), MAP_SCALE*mapbuf.get_height(), gtk.gdk.INTERP_NEAREST)
 	map_x = mapbuf.get_width()/2
 	map_y = mapbuf.get_height()/2
 
 	win = hildon.StackableWindow()
-	win.set_title("Tube route")
+	win.set_title("Tubes")
 	vbox = gtk.VBox()
 	title = gtk.Label()
 	title.station_from = station_from
 	title.station_to = station_to
-	title.set_markup("<big>No route</big>")
+	title.set_markup("<big>Welcome</big>")
 	vbox.pack_start(title, True, True, 5)
 	route = hildon.TextView()
 	route.set_editable(False)
 	route.set_wrap_mode(gtk.WRAP_WORD)
+	route.get_buffer().set_text("Welcome to Tubes, the simple metro router for Maemo. Click one of the buttons below to get started. \"Choose route\" allows you to choose your starting point and destination graphically; click \"Enter route\" if you prefer text entry. The bottom button pops up a handy tube map.\nAll the routing and closures data used in this program is from the TfL website. The tube map is copyright TfL.")
 	route.plan = None
 	hbox = gtk.HBox()
-	routebutton_text = gtk.Button("Enter route (text)")
-	routebutton_text.connect("clicked", lambda button: showdialog(title, route))
 	routebutton_map = gtk.Button("Choose route (map)")
 	routebutton_map.connect("clicked", lambda button: togglemap(mapbuf, map_x, map_y, title, route, True))
-	hbox.pack_start(routebutton_text, True, True, 5)
-	hbox.pack_end(routebutton_map, True, True, 5)
+	routebutton_text = gtk.Button("Enter route (text)")
+	routebutton_text.connect("clicked", lambda button: showdialog(title, route))
+	hbox.pack_start(routebutton_map, True, True, 5)
+	hbox.pack_end(routebutton_text, True, True, 5)
 	mapbutton = gtk.Button("Tube map (double-click to return)")
 	mapbutton.connect("clicked", lambda button: togglemap(mapbuf, map_x, map_y, title, route))
 	vbox.pack_start(route, True, True, 5)
-	vbox.pack_start(gtk.Label(""), True, True, 5)
 	vbox.pack_start(hbox, True, True, 5)
 	vbox.pack_end(mapbutton, True, True, 5)
 	pannable_area = hildon.PannableArea()
